package healthwatcher.aspects.concurrency;

import healthwatcher.data.rdb.ComplaintRepositoryRDB;
import healthwatcher.model.complaint.Complaint;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import lib.exceptions.ExceptionMessages;
import lib.exceptions.PersistenceMechanismException;
import lib.exceptions.ObjectNotFoundException;
import lib.exceptions.PersistenceMechanismException;
import lib.exceptions.RepositoryException;
import lib.persistence.PersistenceMechanism;

import edu.iastate.cs.ptolemy.runtime.*;
import healthwatcher.aspects.concurrency.events.*;

/**
 * Timestamping for the Complaint classes.
 * Introduces the needed methods and creates advice to check inserts, updates and searches
 * 
 */
@Instantiation(Instantiation.Model.SINGLETON)
@Registration(Registration.Model.IMPLICIT)
public class HWTimestamp {

	// Declares the fields and methods necessary for timestamps in Complaints
	private long Complaint.timestamp; // para tratamento de concorrencia (scbs)
	
	public long Complaint.getTimestamp() {
		return timestamp;
	}

	public void Complaint.setTimestamp(long timestamp) {
		this.timestamp = timestamp;
	}

	public void Complaint.incTimestamp() {
		this.timestamp = timestamp + 1;
	}
	
	// After inserting, update the DB
	public void handler(InsertComplaintEvent next) throws Throwable {
		invoke(next);
		updateTimestamp(next.complaint.getTimestamp() + "", "SCBS_queixa", next.complaint.getCodigo() + "");
	}
	when InsertComplaintEvent do handler;
	
	// After searching, load from the DB
//	after(Complaint complaint) : execution(* ComplaintRepositoryRDB.search(int)) && args(complaint) {
	public void handler(SearchComplaintEvent next) throws Throwable {
		invoke(next);
		long timestamp = searchTimestamp("SCBS_queixa", next.complaint.getCodigo() + "");
		next.complaint.setTimestamp(timestamp);
	}
	when SearchComplaintEvent do handler;
	
	// When updating, check the TS before and update DB after
	public void handler(UpdateComplaintEvent next) throws Throwable {
		
		synchronized (this) {
			String sql = null;
			try {
				long timestamp;
				Statement stmt = (Statement) PersistenceMechanism.getInstance().getCommunicationChannel();
				// vendo se a vers�o do objeto � a mesma no BD
				sql = "select ts from SCBS_queixa " + " where codigo='"
						+ next.complaint.getCodigo() + "'";
				ResultSet resultSet = stmt.executeQuery(sql);
				if (resultSet.next()) {
					timestamp = (new Long(resultSet.getString("ts"))).longValue();
					if (timestamp != next.complaint.getTimestamp()) {
							new RepositoryException(
								ExceptionMessages.EXC_FALHA_ATUALIZACAO_COPIA);
					} else {
						next.complaint.incTimestamp();
					}
				} else {
					new ObjectNotFoundException(
						ExceptionMessages.EXC_FALHA_ATUALIZACAO);
				}
				resultSet.close();
				stmt.close();
			} catch (SQLException e) {
				System.out.println(sql);
					new RepositoryException(ExceptionMessages.EXC_FALHA_BD);
			} catch (RepositoryException e) {
					new RepositoryException(ExceptionMessages.EXC_FALHA_ATUALIZACAO);
			}
			try {
				invoke(next);
			} catch (Exception e) {
				e.printStackTrace();
				throw new RuntimeException(e);
			}
			updateTimestamp(next.complaint.getTimestamp() + "", "SCBS_queixa", next.complaint.getCodigo()
					+ "");
		}
	}
	when UpdateComplaintEvent do handler;
	
	// Auxiliary Methods
	private void updateTimestamp(String value, String tableName, String id) {
		Statement stmt = null;
		int result = 0;
		try {
			String sql = "update " + tableName + " set ts='" + value + "' where codigo='" + id
					+ "'";
			stmt = (Statement) PersistenceMechanism.getInstance().getCommunicationChannel();
			result = stmt.executeUpdate(sql);
			if (result == 0) {
				throw new RuntimeException("ERRO no aspecto TimestampAspectHealthWatcher ##2");
			}
		} catch (Exception ex) {

			ex.printStackTrace();
			throw new RuntimeException(ex);
		} finally {
			try {
				if (stmt != null)
					stmt.close();
			} catch (Exception ex) {
				throw new RuntimeException(ex);
			}
		}
	}

	private long searchTimestamp(String tableName, String id) {
		Statement stmt = null;
		ResultSet resultSet = null;
		long answer = 0;
		try {
			String sql = "SELECT ts FROM " + tableName + " WHERE codigo='" + id + "'";

			PersistenceMechanism pm = PersistenceMechanism.getInstance();
			stmt = (Statement) pm.getCommunicationChannel();
			resultSet = stmt.executeQuery(sql);
			if (resultSet.next()) {
				answer = resultSet.getLong(1);
			} else {
				throw new RuntimeException("ERRO no aspecto TimestampAspectHealthWatcher ##2");
			}
			return answer;
		} catch (Exception ex) {
			ex.printStackTrace();
			throw new RuntimeException(ex);
		} finally {
			try {
				if (resultSet != null)
					resultSet.close();
				if (stmt != null)
					stmt.close();
			} catch (Exception ex) {
				throw new RuntimeException(ex);
			}
		}
	}
	
	/**
	 * This advice is needed because of distribution. When the client side makes an update, the
	 * server side complaint is updated, but the client side is not. Thus, we need also to update
	 * the client side object (hence the cflow) and increment it.
	 * Note that this was not needed before the Observer pattern, because we would update the object
	 * just once per request. Now we update it more than once and this synchronization is needed.
	 */
	int cflowCounter = 0;
	public void handler(ExecuteCommandEvent next) throws Throwable {
		cflowCounter++;
		invoke(next);
		cflowCounter--;
	}
	when ExecuteCommandEvent do handler;

	public void handler(ObserverUpdatedEvent next) throws Throwable {
		
		invoke(next);
		if (cflowCounter < 1)
			return;
		next.c.incTimestamp();
	}
	when ObserverUpdatedEvent do handler;
}
