package healthwatcher.aspects.exceptionHandling;

import healthwatcher.data.rdb.AddressRepositoryRDB;
import healthwatcher.data.rdb.ComplaintRepositoryRDB;
import healthwatcher.model.address.Address;
import healthwatcher.model.complaint.Complaint;
import healthwatcher.view.servlets.HWServlet;

import java.io.IOException;
import java.io.PrintWriter;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import lib.exceptions.ConnectionPersistenceMechanismException;
import lib.exceptions.ExceptionMessages;
import lib.exceptions.PersistenceMechanismException;
import lib.exceptions.RepositoryException;
import lib.exceptions.SQLPersistenceMechanismException;
import lib.persistence.PersistenceMechanism;
import lib.util.HTMLCode;

import org.aspectj.lang.SoftException;

import edu.iastate.cs.ptolemy.runtime.*;
import healthwatcher.aspects.exceptionHandling.events.*;
import healthwatcher.aspects.concurrency.events.*;

/**
 * This aspect handles repository exceptions in the servlets
 */
@Instantiation(Instantiation.Model.SINGLETON)
@Registration(Registration.Model.IMPLICIT)
@PrecedesHandler(healthwatcher.aspects.patterns.ServletCommanding.class)
public class HWPersistenceExceptionHandler {

	Map persistenceLocalVariable = new HashMap();

	public void handler(HWServletExEvent next) throws Throwable {

		try {

			invoke(next);

		} catch (SoftException e) {
			if (e.getWrappedThrowable() instanceof RepositoryException) {
				PrintWriter out = next.response.getWriter();
				out.println(HTMLCode.errorPage(e.getWrappedThrowable()
						.getMessage()));
				out.close();
			} else {
				throw e;
			}
		}
	}
	when HWServletExEvent do handler;

	//	----------------------------------AddressRepositoryRDB------------------------
	public void handler(SearchQueryExecutedEvent next) throws Throwable {
		persistenceLocalVariable.put(String.valueOf(Thread.currentThread().getId()), next.sql);
		invoke(next);
	}
	when SearchQueryExecutedEvent do handler;

	public void handler(SearchAddressEvent next) throws Throwable {
		try {
			invoke(next);
		} catch (PersistenceMechanismException e) {
			throw new RepositoryException(ExceptionMessages.EXC_FALHA_BD);
		} catch (SQLException e) {
			throw new SQLPersistenceMechanismException(ExceptionMessages.EXC_FALHA_BD,
					(String) persistenceLocalVariable.get(String
							.valueOf(Thread.currentThread().getId())));
		}
	}
	when SearchAddressEvent do handler;
	
	//----------------------------------ComplaintRepositoryRDB.update------------------------
	
	public void handler(UpdateComplaintEvent next) throws Throwable {
		try {
			invoke(next);
		} catch (PersistenceMechanismException e) {
			throw new RepositoryException(ExceptionMessages.EXC_FALHA_ATUALIZACAO);
		}
	}
	when UpdateComplaintEvent do handler;
	
	public void handler(InternalUpdateQueryExecutedEvent next) throws Throwable {
		persistenceLocalVariable.put(String.valueOf(Thread.currentThread().getId()), next.sql);
		invoke(next);
	}
	when InternalUpdateQueryExecutedEvent do handler;

	public void handler(InternalUpdateEvent next) throws Throwable {
		try {
			invoke(next);
		} catch (SQLException e) {
			throw new SQLPersistenceMechanismException(ExceptionMessages.EXC_FALHA_BD,
					(String) persistenceLocalVariable.get(String
							.valueOf(Thread.currentThread().getId())));
		}
	}
	when InternalUpdateEvent do handler;
	
	//	----------------------------------ComplaintRepositoryRDB.insert------------------------
	
	public void handler(InsertComplaintEvent next) throws Throwable {
		try {
			invoke(next);
		} catch (PersistenceMechanismException e) {
			throw new RepositoryException(ExceptionMessages.EXC_FALHA_INCLUSAO);
		} catch (SQLException e) {
			throw new SQLPersistenceMechanismException(ExceptionMessages.EXC_FALHA_BD,
					(String) persistenceLocalVariable.get(String
							.valueOf(Thread.currentThread().getId())));
		}
	}
	when InsertComplaintEvent do handler;
	
	public void handler(InsertQueryExecutedEvent next) throws Throwable {
		persistenceLocalVariable.put(String.valueOf(Thread.currentThread().getId()), next.consulta);
		invoke(next);
	}
	when InsertQueryExecutedEvent do handler;
	

	// ----------------------------------PersistenceMechanism.connect---------------------------------
	
	public void handler(ConnectEvent next) throws Throwable {
		
		try {
			invoke(next);
		} catch (Exception e) {
			next.obj.incCurrentMirror();
			try {
				next.obj.connect();
			} catch (Exception ex1) {
				ex1.printStackTrace();
				throw new ConnectionPersistenceMechanismException(
						ExceptionMessages.EXC_FALHA_GET_CANAL_COMUNICACAO, next.obj.getUrl());
			}
		}
	}
	when ConnectEvent do handler;

	//----------------------------------PersistenceMechanism.getCommunicationChannel()---------------------------------
	
	public Object handler(GetCommChannelEvent next) throws Throwable {
		
		try {
			return invoke(next);
		} catch (Exception e) {
			next.obj.incCurrentMirror();
			try {
				next.obj.connect();
				return next.obj.getCommunicationChannel(false).createStatement();
			} catch (Exception ex1) {
				ex1.printStackTrace();
				throw new ConnectionPersistenceMechanismException(
						ExceptionMessages.EXC_FALHA_GET_CANAL_COMUNICACAO, next.obj.getUrl());
			}
		}
	}
	when GetCommChannelEvent do handler;
}
