package healthwatcher.aspects.patterns;

import healthwatcher.Constants;
import healthwatcher.data.factories.AbstractRepositoryFactory;
import healthwatcher.data.factories.ArrayRepositoryFactory;
import healthwatcher.data.factories.RDBRepositoryFactory;

import healthwatcher.aspects.persistence.*;

import edu.iastate.cs.ptolemy.runtime.*;

/**
 * This aspect provides the factory according to the system's configuration
 */
@Instantiation(Instantiation.Model.SINGLETON)
public class RepositoryFactory {

	/**
	 * The repository factory singleton instance
	 */
	private static AbstractRepositoryFactory AbstractRepositoryFactory.instance = null;
	
	/**
	 * Gets the repository factory configured for the Health Watcher system,
	 * creating the instance if needed
	 * 
	 * @return the configured repository factory
	 */
	public static AbstractRepositoryFactory AbstractRepositoryFactory.getRepositoryFactory() {
		if (instance == null) {
			if (Constants.isPersistent()) {
		 		// PTOLEMY FIXME - how do we get the singleton instance?
//				instance = new RDBRepositoryFactory(HWPersistence.aspectOf().getPm());
			} else {
				instance = new ArrayRepositoryFactory();
			}
		}
		return instance;
	}	
}
