package healthwatcher.aspects.patterns;

import healthwatcher.business.HealthWatcherFacade;
import healthwatcher.model.complaint.Complaint;
import healthwatcher.model.complaint.Symptom;
import healthwatcher.model.employee.Employee;
import healthwatcher.model.healthguide.HealthUnit;
import healthwatcher.model.healthguide.MedicalSpeciality;
import healthwatcher.view.IFacade;
import healthwatcher.view.command.CommandRequest;
import healthwatcher.view.command.CommandServlet;
import lib.patterns.ObserverProtocol;

import edu.iastate.cs.ptolemy.runtime.*;
import healthwatcher.aspects.patterns.events.*;
import healthwatcher.aspects.concurrency.events.*;
import lib.patterns.*;


/**
 * Implements the observer protocol to update automatically to the DB changes made in model
 * objects. This aspect also is responsible by adding the observers when the subject objects
 * are created.
 */
@Instantiation(Instantiation.Model.SINGLETON)
@Registration(Registration.Model.IMPLICIT)
public class UpdateStateObserver extends ObserverProtocol {
	
	declare parents: IFacade implements Observer;

	declare parents: Complaint implements Subject;
	declare parents: Employee implements Subject;
	declare parents: Symptom implements Subject;
	declare parents: MedicalSpeciality implements Subject;
	declare parents: HealthUnit implements Subject;
	
	public void handler(PutSubjectEvent next) throws Throwable {
		invoke(next);
		addObserver(next.s, HealthWatcherFacade.getInstance());
	}
	when PutSubjectEvent do handler;
	
	protected void updateObserver(Subject subject, Observer observer) {
		if(observer instanceof IFacade) {
			IFacade facade= (IFacade) observer;
			
			if(subject instanceof Employee){
				facade.updateEmployee((Employee) subject);
			} else if(subject instanceof Complaint){
				announce ObserverUpdatedEvent((Complaint) subject) {
					facade.updateComplaint((Complaint) subject);
				}
			} else if(subject instanceof HealthUnit){
				facade.updateHealthUnit((HealthUnit) subject);
			} else if(subject instanceof Symptom){
				facade.updateSymptom((Symptom) subject);
			} else if(subject instanceof MedicalSpeciality){
				facade.updateMedicalSpeciality((MedicalSpeciality) subject);
			}
		}
	}

}
