package healthwatcher.aspects.patterns.complaintState;

/**
 * Implements the state transitions for this state design pattern example. 
 * State transitions are realizied as <code>after</code> advice. The 
 * joinpoints are the calls from the context to its state object.<p>
 *
 * Exisiting states are reused without a employing a flyweight mechanism or 
 * (inflexibly) modularizing the transitions in the context.
 *
 * @author  Marcos D�sea
 * @author  S�rgio Soares
 * 
 */

import healthwatcher.model.address.Address;
import healthwatcher.model.complaint.Complaint;
import healthwatcher.model.complaint.Situation;
import healthwatcher.model.complaint.state.ComplaintState;
import healthwatcher.model.complaint.state.ComplaintStateClosed;
import healthwatcher.model.complaint.state.ComplaintStateOpen;
import healthwatcher.model.employee.Employee;
import lib.util.Date;

import edu.iastate.cs.ptolemy.runtime.*;
import healthwatcher.aspects.patterns.events.*;



@Instantiation(Instantiation.Model.SINGLETON)
@Registration(Registration.Model.IMPLICIT)
public class ComplaintStateAspect {
    
	public void handler(NewOpenComplaintEvent next) throws Throwable {
    	invoke(next);
		next.complaint.setComplaintState( new ComplaintStateOpen() );
	}
	when NewOpenComplaintEvent do handler;
	
	public void handler(NewComplaintEvent next) throws Throwable {
    	 invoke(next);
		 if(next.status==Situation.QUEIXA_ABERTA) {
			 next.complaint.setComplaintState( new ComplaintStateOpen(0, next.complainer, next.description, next.observation, next.email, next.attendant,
					 next.status, next.medicalOpinionDate, next.complaintDate, next.complainerAddress));
		 } else if(next.status==Situation.QUEIXA_FECHADA) {
			 next.complaint.setComplaintState( new ComplaintStateClosed(0, next.complainer, next.description, next.observation, next.email, next.attendant,
					 next.status, next.medicalOpinionDate, next.complaintDate, next.complainerAddress));
		 }
	}
	when NewComplaintEvent do handler;
	
	public void handler(ComplaintSetStatusEvent next) throws Throwable {
    	invoke(next);
		
		if (next.status != next.state.getStatus()) {
			if (next.status == Situation.QUEIXA_ABERTA) {
				next.complaint.setComplaintState(new ComplaintStateOpen(next.state.getCode(),
						next.state.getComplainer(), next.state.getDescription(), next.state.getObservation(), 
						next.state.getEmail(), next.state.getAttendant(),	next.state.getStatus(), next.state.getMedicalOpinionDate(), 
						next.state.getComplaintDate(), next.state.getComplainerAddress()));
			} else if (next.status == Situation.QUEIXA_FECHADA) {
				next.complaint.setComplaintState(new ComplaintStateClosed(next.state.getCode(),
						next.state.getComplainer(), next.state.getDescription(), next.state.getObservation(), 
						next.state.getEmail(), next.state.getAttendant(),	next.state.getStatus(), next.state.getMedicalOpinionDate(), 
						next.state.getComplaintDate(), next.state.getComplainerAddress()));
			} else if (next.status == Situation.QUEIXA_SUSPENSA) {

			}
		}
	}
	when ComplaintSetStatusEvent do handler;

}