package healthwatcher.aspects.persistence;

import healthwatcher.Constants;
import healthwatcher.business.complaint.ComplaintRecord;
import healthwatcher.business.complaint.DiseaseRecord;
import healthwatcher.business.complaint.SymptomRecord;
import healthwatcher.business.employee.EmployeeRecord;
import healthwatcher.business.healthguide.HealthUnitRecord;
import healthwatcher.business.healthguide.MedicalSpecialityRecord;
import healthwatcher.data.factories.AbstractRepositoryFactory;
import lib.exceptions.RepositoryException;
import lib.persistence.IPersistenceMechanism;

import edu.iastate.cs.ptolemy.runtime.*;
import healthwatcher.aspects.persistence.events.*;

/**
 * This aspect intercepts Record creations and inserts Records with the proper repository
 * instance, depending if it is persistent or not
 */
@Instantiation(Instantiation.Model.SINGLETON)
@Registration(Registration.Model.IMPLICIT)
public class HWDataCollection {
	
	public Object handler(SystemRecordMethodEvent next) throws Throwable {
		return getSystemRecord(next.type);
	}
	when SystemRecordMethodEvent do handler;
	

	/**
	 * Creates a system record depending on the class type
	 */
	protected Object getSystemRecord(Class type) {

		AbstractRepositoryFactory factory = AbstractRepositoryFactory.getRepositoryFactory();
		
		if (type.equals(ComplaintRecord.class)) {
			return new ComplaintRecord(factory.createComplaintRepository());
		} else  if (type.equals(HealthUnitRecord.class)) {
			return new HealthUnitRecord(factory.createHealthUnitRepository());
		} else  if (type.equals(MedicalSpecialityRecord.class)) {
			return new MedicalSpecialityRecord(factory.createMedicalSpecialityRepository());
		} else  if (type.equals(EmployeeRecord.class)) {
			return new EmployeeRecord(factory.createEmployeeRepository());
		} else if(type.equals(DiseaseRecord.class)){
			return new DiseaseRecord(factory.createDiseaseRepository());
		} else if(type.equals(SymptomRecord.class)){
			return new SymptomRecord(factory.createSymptomRepository());
		}
		return null;
	}

 	/**
 	 * Returns the persistence mechanism in use
 	 */
 	protected IPersistenceMechanism getPm() {
 		// PTOLEMY FIXME - how do we get the singleton instance?
// 		return HWPersistence.aspectOf().getPm();
 		return null;
 	}
}
