/**
 * Copyright (c) 2006 Macacos.org. All Rights Reserved
 *
 * This library is free software; you can redistribute it and/or modify it under the terms of the 
 * GNU Lesser General Public License as published by the Free Software Foundation; either version 
 * 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See 
 * the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with this library;
 * if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, 
 * MA  02110-1301  USA
 * 
 * You can find the license also here: http://www.gnu.org/copyleft/lesser.html
 * 
 * 
 * Created on Sep 17, 2006 by Thiago Tonelli Bartolomei
 * -------------------------------------------------
 *           \                                                                           
 *            \                                                                          
 *               __                                                                      
 *          w  c(..)o                                                                    
 *           \__(o)                                                                      
 *               /\                                                                      
 *            w_/(_)-~                                                                   
 *                /|                                                                     
 *               | \                                                                     
 *               m  m   
 */
package healthwatcher.aspects.persistence;

import healthwatcher.business.HealthWatcherFacade;
import lib.exceptions.TransactionException;
import lib.persistence.IPersistenceMechanism;

import edu.iastate.cs.ptolemy.runtime.*;
import healthwatcher.aspects.persistence.events.*;

/**
 * This aspect makes all methods in the facade transactionals and softens
 * the transaction exceptions in the persistence mechanism
 */
@Instantiation(Instantiation.Model.SINGLETON)
@Registration(Registration.Model.IMPLICIT)
public class HWTransactionManagement {

	// PTOLEMY FIXME - how do we exclude GetInstanceEvent??
	public Object handler(HWFacadeMethodEvent next) throws Throwable {
        getPm().beginTransaction();
		try {
			Object ret = invoke (next);
	        getPm().commitTransaction();
			return ret;      
		} catch (Exception e) {
	        getPm().rollbackTransaction();
			throw e;
		}
	}
	when HWFacadeMethodEvent do handler;

	public void handler(HWFacadeVoidMethodEvent next) throws Throwable {
        getPm().beginTransaction();
		try {
			invoke (next);
	        getPm().commitTransaction();
		} catch (Exception e) {
	        getPm().rollbackTransaction();
			throw e;
		}
	}
	when HWFacadeVoidMethodEvent do handler;

    public IPersistenceMechanism getPm() {
		// PTOLEMY FIXME - how do we get the singleton instance?
//		return HWPersistence.aspectOf().getPm();
		return null;
	}
}
