package healthwatcher.business;

import healthwatcher.business.complaint.ComplaintRecord;
import healthwatcher.business.complaint.DiseaseRecord;
import healthwatcher.business.complaint.SymptomRecord;
import healthwatcher.business.employee.EmployeeRecord;
import healthwatcher.business.healthguide.HealthUnitRecord;
import healthwatcher.business.healthguide.MedicalSpecialityRecord;
import healthwatcher.model.complaint.Complaint;
import healthwatcher.model.complaint.DiseaseType;
import healthwatcher.model.complaint.Symptom;
import healthwatcher.model.employee.Employee;
import healthwatcher.model.healthguide.HealthUnit;
import healthwatcher.model.healthguide.MedicalSpeciality;
import healthwatcher.view.IFacade;
import lib.exceptions.InsertEntryException;
import lib.exceptions.ObjectAlreadyInsertedException;
import lib.exceptions.ObjectNotFoundException;
import lib.exceptions.ObjectNotValidException;
import lib.util.IteratorDsk;

import healthwatcher.aspects.persistence.events.*;
import healthwatcher.aspects.distribution.*;

public class HealthWatcherFacade implements IFacade, IRemoteFacade {
	
	private static HealthWatcherFacade singleton; // padrao singleton
	
	private ComplaintRecord complaintRecord;

	private HealthUnitRecord healthUnitRecord;

	private MedicalSpecialityRecord specialityRecord;

	private DiseaseRecord diseaseRecord;

	private EmployeeRecord employeeRecord;

	private SymptomRecord symptomRecord;
	
	private HealthWatcherFacade() {
		// PTOLEMY FIXME - why is the compiler complaining here... ?
//		this.complaintRecord = announce SystemRecordMethodEvent(ComplaintRecord.class) { return new ComplaintRecord(null); };
//		this.healthUnitRecord = announce SystemRecordMethodEvent(HealthUnitRecord.class) { return new HealthUnitRecord(null); };
//		this.specialityRecord = announce SystemRecordMethodEvent(MedicalSpecialityRecord.class) { return new MedicalSpecialityRecord(null); };
//		this.diseaseRecord = announce SystemRecordMethodEvent(DiseaseRecord.class) { return new DiseaseRecord(null); };
//		this.employeeRecord = announce SystemRecordMethodEvent(EmployeeRecord.class) { return new EmployeeRecord(null); };
//		this.symptomRecord = announce SystemRecordMethodEvent(SymptomRecord.class) { return new SymptomRecord(null); };
	}

	public synchronized static HealthWatcherFacade getInstance() {
		announce GetHWInstanceEvent() {
			if (singleton == null) {
				announce NewHWFacadeEvent() {
					singleton = new HealthWatcherFacade();
				}
			}
			return singleton;
		}
	}
			
	public void updateHealthUnit(HealthUnit unit) throws ObjectNotFoundException, ObjectNotValidException {
		announce HWFacadeVoidMethodEvent() {
			healthUnitRecord.update(unit);
		}
	}

	public void updateComplaint(Complaint complaint) throws ObjectNotFoundException, ObjectNotValidException {
		announce HWFacadeVoidMethodEvent() {
			complaintRecord.update(complaint);
		}
	}

	public IteratorDsk searchSpecialitiesByHealthUnit(int code) throws ObjectNotFoundException {
		announce HWFacadeMethodEvent() {
			return healthUnitRecord.searchSpecialityByHealthUnit(code);
		}
	}

	public Complaint searchComplaint(int code) throws ObjectNotFoundException {
		announce HWFacadeMethodEvent() {
			return complaintRecord.search(code);
		}
	}

	public DiseaseType searchDiseaseType(int code) throws ObjectNotFoundException {
		announce HWFacadeMethodEvent() {
			return diseaseRecord.search(code);
		}
	}

	public IteratorDsk searchHealthUnitsBySpeciality(int code) throws ObjectNotFoundException {
		announce HWFacadeMethodEvent() {
			return healthUnitRecord.searchHealthUnitsBySpeciality(code);
		}
	}

	public IteratorDsk getSpecialityList() throws ObjectNotFoundException {
		announce HWFacadeMethodEvent() {
			return specialityRecord.getListaEspecialidade();
		}
	}

	public IteratorDsk getDiseaseTypeList() throws ObjectNotFoundException {
		announce HWFacadeMethodEvent() {
			return diseaseRecord.getDiseaseTypeList();
		}
	}

	public HealthUnit searchHealthUnit(int healthUnitCode) throws ObjectNotFoundException {
		announce HWFacadeMethodEvent() {
			return healthUnitRecord.search(healthUnitCode);
		}
	}

	public IteratorDsk getHealthUnitList() throws ObjectNotFoundException {
		announce HWFacadeMethodEvent() {
			return healthUnitRecord.getHealthUnitList();
		}
	}

	public IteratorDsk getPartialHealthUnitList() throws ObjectNotFoundException {
		announce HWFacadeMethodEvent() {
			return healthUnitRecord.getPartialHealthUnitList();
		}
	}

	public void insert(Employee employee) throws ObjectAlreadyInsertedException,
			ObjectNotValidException {
		announce HWFacadeVoidMethodEvent() {
			employeeRecord.insert(employee);
		}
	}

	public int insertComplaint(Complaint complaint) throws ObjectAlreadyInsertedException, ObjectNotValidException {
		announce HWFacadeMethodEvent() {
			return complaintRecord.insert(complaint);
		}
	}

	public Employee searchEmployee(String login) throws ObjectNotFoundException {
		announce HWFacadeMethodEvent() {
			return employeeRecord.search(login);
		}
	}

	public IteratorDsk getComplaintList() throws ObjectNotFoundException {
		announce HWFacadeMethodEvent() {
			return complaintRecord.getComplaintList();
		}
	}

	public void updateEmployee(Employee employee) throws ObjectNotFoundException,
			ObjectNotValidException {
		announce HWFacadeVoidMethodEvent() {
			employeeRecord.update(employee);
		}
	}
	
	public void insert(HealthUnit us) throws InsertEntryException, ObjectAlreadyInsertedException,
			ObjectNotValidException {
		announce HWFacadeVoidMethodEvent() {
			healthUnitRecord.insert(us);
		}
	}

	public void insert(MedicalSpeciality speciality) throws ObjectAlreadyInsertedException,
			ObjectNotValidException, InsertEntryException {
		announce HWFacadeVoidMethodEvent() {
			specialityRecord.insert(speciality);
		}
	}

	public void insert(Symptom symptom) throws ObjectAlreadyInsertedException,
			InsertEntryException, ObjectNotValidException {
		announce HWFacadeVoidMethodEvent() {
			symptomRecord.insert(symptom);
		}
	}

	public Symptom searchSymptom(int numSymptom) throws ObjectNotFoundException {
		announce HWFacadeMethodEvent() {
			return symptomRecord.search(numSymptom);
		}
	}

	public IteratorDsk getSymptomList() throws ObjectNotFoundException {
		announce HWFacadeMethodEvent() {
			return symptomRecord.getSymptomList();
		}
	}

	public void updateSymptom(Symptom symptom) throws ObjectNotFoundException,
			ObjectNotValidException {
		announce HWFacadeVoidMethodEvent() {
			symptomRecord.update(symptom);
		}
	}

	public MedicalSpeciality searchSpecialitiesByCode(int numSpeciality)
			throws ObjectNotFoundException {
		announce HWFacadeMethodEvent() {
			return specialityRecord.search(numSpeciality);
		}
	}

	public void updateMedicalSpeciality(MedicalSpeciality speciality)
			throws ObjectNotFoundException, ObjectNotValidException {
		announce HWFacadeVoidMethodEvent() {
			specialityRecord.update(speciality);
		}
	}

	public void insert(DiseaseType diseaseType) throws InsertEntryException,
			ObjectAlreadyInsertedException, ObjectNotValidException {
		announce HWFacadeVoidMethodEvent() {
			diseaseRecord.insert(diseaseType);
		}
	}
}