package healthwatcher.aspects.distribution;

import java.rmi.Remote;
import java.rmi.RemoteException;

import lib.distribution.rmi.IteratorRMISourceAdapter;
import lib.distribution.rmi.IteratorRMITargetAdapter;
import lib.distribution.rmi.MethodExecutor;
import lib.util.LocalIterator;

import edu.iastate.cs.ptolemy.runtime.*;
import healthwatcher.aspects.distribution.events.*;

/**
 * A concrete distribution aspect for HW clients, using RMI.
 * 
 */
@Instantiation(Instantiation.Model.SINGLETON)
@Registration(Registration.Model.IMPLICIT)
public class RMIClientDistribution extends HWClientDistribution {

	/**
	 * Keeps the remote instance
	 */
	protected IRemoteFacade facade = null;
	
	/**
     * Redirects the facade methods call to the distributed instance methods using reflection
     */
	public Object handler (IFacadeMethodEvent next) {
	    Object obj = MethodExecutor.invokeMethod(
		    	getRemoteFacade(), next.name, next.args);
		    if (obj instanceof LocalIterator) {
				IteratorRMITargetAdapter iteratorTA = new IteratorRMITargetAdapter((LocalIterator) obj, 3);
				return new IteratorRMISourceAdapter(iteratorTA, (LocalIterator) obj, 3);
		    }
		    return obj;
	}
	when IFacadeMethodEvent do handler;

	public void handler (IFacadeVoidMethodEvent next) {
		
		Object obj = null;
		
		try {
		    obj = MethodExecutor.invokeMethod(
			    	getRemoteFacade(), next.name, next.args);

		} catch (RuntimeException e) {
			if (e.getCause() instanceof RemoteException) {
				obj = retry(next.name, next.args);
			}
    	} catch (Throwable e) {
    		e.printStackTrace();
    	}    	
	    
	    if (obj instanceof LocalIterator) {
			IteratorRMITargetAdapter iteratorTA = new IteratorRMITargetAdapter((LocalIterator) obj, 3);
			new IteratorRMISourceAdapter(iteratorTA, (LocalIterator) obj, 3);
	    }
	}
	when IFacadeVoidMethodEvent do handler;
    
	/**
	 * Tries again. Makes facade null to rebind the facade
	 */
	protected Object retry(String methodName, Object[] params) {
		try {
			facade = null;
			return MethodExecutor.invokeMethod(
					getRemoteFacade(), methodName, params);
		} catch (Throwable t) {
			t.printStackTrace();
			return null;
		}
	}
	
    /**
     * Retrieves the remote instance, creating if needed
     */
	private synchronized Remote getRemoteFacade() {        
		if (facade == null) {
			try {
	            System.out.println("About to lookup...");
	            facade = (IRemoteFacade) java.rmi.Naming.lookup("//" + healthwatcher.Constants.SERVER_NAME + "/" + healthwatcher.Constants.SYSTEM_NAME);
	            System.out.println("Remote DisqueSaude found");
	        } catch (java.rmi.RemoteException rmiEx) {
	            initExceptionHandling(rmiEx);
	        } catch (java.rmi.NotBoundException rmiEx) {
	            initExceptionHandling(rmiEx);
	        } catch (java.net.MalformedURLException rmiEx) {
	            initExceptionHandling(rmiEx);
	        }
		}
		return facade;
    }
}
